


import unittest
from nlu import *

class TestSentiment(unittest.TestCase):
    def test_sentiment_model(self):
        pipe = nlu.load('sentiment',verbose=True)
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='document',drop_irrelevant_cols=False, metadata=True, )
        for c in df.columns: print(df[c])

    def test_sentiment_imdb_model(self):
        pipe = nlu.load('sentiment.twitter',verbose=True)
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'])#, output_level='document',drop_irrelevant_cols=False, metadata=True, )
        print(df.columns)
        for c in df.columns: print(df[c])
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='document')
        print(df.columns)
        for c in df.columns: print(df[c])



    def test_sentiment_twitter_model(self):
        pipe = nlu.load('sentiment.imdb',verbose=True)
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='sentence')
        print(df.columns)
        for c in df.columns: print(df[c])
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='document')
        print(df.columns)
        for c in df.columns: print(df[c])


    def test_sentiment_detector_model(self):
        pipe = nlu.load('sentiment.imdb',verbose=True)
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='sentence')
        for c in df.columns: print(df[c])
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='document')
        for c in df.columns: print(df[c])

    def test_sentiment_vivk_model(self):
        pipe = nlu.load('sentiment.vivekn',verbose=True)
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='sentence')
        print(df.columns)
        for c in df.columns: print(df[c])
        df = pipe.predict(['I love pancaces. I hate Mondays', 'I love Fridays'], output_level='document')
        for c in df.columns: print(df[c])


if __name__ == '__main__':
    unittest.main()




import logging
logger = logging.getLogger('nlu')
import nlu
import requests
class ModelHubUtils():
    modelhub_json_url = 'https://nlp.johnsnowlabs.com/models.json'
    data = requests.get(modelhub_json_url).json()
    """Modelhub utils"""
    @staticmethod
    def NLU_ref_to_NLP_ref(nlu_ref: str,lang: str = None) -> str:
        """Resolve a Spark NLU reference to q NLP reference.
        Args :
        NLU_ref : which nlu model's nlp refrence to return.
        lang : what language is the model in.
        """
        nlu_namespaces_to_check = [nlu.Spellbook.pretrained_pipe_references, nlu.Spellbook.pretrained_models_references, nlu.Spellbook.pretrained_healthcare_model_references, nlu.Spellbook.licensed_storage_ref_2_nlu_ref , nlu.Spellbook.storage_ref_2_nlu_ref]#]
        for dict_ in nlu_namespaces_to_check:
            if lang:
                if lang in dict_.keys():
                    for reference in dict_[lang]:
                        if reference ==nlu_ref:
                            return  dict_[lang][reference]
            else :
                for dict_ in nlu_namespaces_to_check:
                    for lang in dict_:
                        for reference in dict_[lang]:
                            if reference ==nlu_ref:
                                return  dict_[lang][reference]
        for _nlp_ref, nlp_ref_type in nlu.Spellbook.component_alias_references.items():
            if _nlp_ref == nlu_ref : return  nlp_ref_type[0]

        return ''


    @staticmethod
    def get_url_by_nlu_refrence(nlu_refrence: str ) -> str:
        """Rsolves  a  URL for an NLU refrence.

        Args :
            nlu_refrence : Which nlu refrence's url to return.

        """

        # getting spark refrence for given nlu refrence
        if nlu_refrence =='' : return  'https://nlp.johnsnowlabs.com/models'
        if nlu_refrence.split(".")[0] not in  nlu.AllComponentsInfo().all_languages:
            nlu_refrence = "en."+nlu_refrence
        nlp_refrence= ModelHubUtils.NLU_ref_to_NLP_ref(nlu_refrence)
        if nlp_refrence == None :
            print(f"{nlp_refrence}         {nlu_refrence}")
            return 'https://nlp.johnsnowlabs.com/models'
        else :
            for model in ModelHubUtils.data :
                if (model['language'] in nlu_refrence.split(".") or model['language'] in nlp_refrence.split('_')) and model['name'] == nlp_refrence:
                    return f"https://nlp.johnsnowlabs.com/{model['url']}"
        return 'https://nlp.johnsnowlabs.com/models'

    @staticmethod
    def return_json_entry(nlu_refrence:str) -> dict:
        """Resolves a Json entry for an nlp_refrence.

        Args:
            nlp_refrence: What nlp_refrence to resolve.
        """

        if nlu_refrence.split(".")[0] not in  nlu.AllComponentsInfo().all_languages:
            nlu_refrence = "en."+nlu_refrence
        nlp_refrence= ModelHubUtils.NLU_ref_to_NLP_ref(nlu_refrence)

        language = nlu_refrence.split(".")[0]
        for model in ModelHubUtils.data :
            if model['language']== language and model["name"] == nlp_refrence:
                return model




from nlu.pipe.pipe_components import SparkNLUComponent
class Deidentification(SparkNLUComponent):
    def __init__(self, annotator_class='deidentifier', lang='en', component_type='deidentifier', get_default=False, model = None, nlp_ref ='', nlu_ref='', trainable=False, is_licensed=True,loaded_from_pretrained_pipe=False):
        annotator_class= 'deidentifier'
        if model != None : self.model = model
        else :
            if annotator_class == 'deidentifier':
                from nlu.components.deidentifiers.deidentifier.deidentifier import Deidentifier
                if get_default : self.model = Deidentifier.get_default_model()
                else : self.model = Deidentifier.get_pretrained_model(nlp_ref, lang)

        SparkNLUComponent.__init__(self, annotator_class, component_type, nlu_ref, nlp_ref, lang,loaded_from_pretrained_pipe , is_licensed)



from typing import List, Dict, Set, Dict, Tuple, Optional, Any,Callable, Dict
from dataclasses import dataclass, field

"""
Contains dataclases which represent Extractors and their configuration

"""


""" Cool ideas but maybe overkill
- What abotu multiple exctracotrs? Make attribute extractor attrivbute a luist of methods? People of can just write phat mehtods? Buy what about combing n
- white/black list based application of specific methods to specirfic keywords . 
- What if you want multiple output cols, one per extractor method? 

"""
from typing import List, Dict, Set, Dict, Tuple, Optional, Any
from typing import Callable
from typing import List, Dict, Set, Dict, Tuple, Optional
from dataclasses import dataclass, field
"""
Contains dataclases which represent Extractors and their configuration

"""

@dataclass
class SparkNLPExtractor:
    """
    Define and describe a **extractor_method** that will be called on every anotators output row.
    The extractor method will recieve a list of dictionaries of type Dict[Str,Any] and must return a List[Any]
    It will recieve a list of annotation results of a particular field for a given row of spark nlp pipeline transformation results.
    Wrap a extractor method, description and name in a dataclass that is used as part of a SparkNLPExtractorConfig

    Parameters
    ----------

    extractor_method    : Callable[[List[Dict[str,Any]]], List[Any]]  :
          An extractor is a method that takes in [Dict[str,Any]] and returs [Any]. Will be applied to every output row's metadata for that particular
    description         : str
          Descrioptioon
    name                :str
          Descrioptioon
    """
    extractor_method    :Callable[[List[Dict[str,Any]]], List[Any]] = field(repr=False, default=lambda x : x)
    description         :str                                        = field(default='')
    name                :str                                        = field(default='')


## TODO extra config for getting "to which sentence did chunk/x/y/z belong to?"
## TODO What abotu PARAMTERIXED extractor methods, like get_k_confidence for lang classifier??
@dataclass
class SparkNLPExtractorConfig:
    """
    Universal Configuration class for defining what data to extract from a Spark NLP annotator.
    These extractor configs can be passed to any extractor NLU defined for Spark-NLP.
    Setting a boolean config to false, results in the extractor NOT returning that field from the Annotator outputs
    Metadata Extractor methods will be applied to metadata after white/black list filtering has been applied to the fields
    Extractor methods for the fields

    output_col_prefix : str
          Prefix used for naming output columns from this annotator this config is applied to/
    description : str
          Describes how this configs affects te outputs
    name : str
          A name for this config
    get_positions : bool
          Get Annotation ending and beginning. If this is set to true, get_begin and get_end will be ignore and both positons will be outputted
    get_begin : bool
          Get Annotation beginnings
    get_end : bool
          Get Annotation ends
    get_embeds : bool
          Get Annotation Embeds
    get_result : bool
          Get Annotation results
    get_meta : bool
          Should get meta any meta at all? IF THIS FALSE, get_full_meta, whitelist and extractors will be ignored.
    get_sentence_origin : bool.
          Should extract from which sentence a prediction was generated from. If output level is Document, this field is irrelevant and should be set to False.
    get_full_meta : bool
          Get all keys and vals from base meta map. If this is true, white/blacklist will be ignored
    get_annotator_type : bool
          Get Annotator Type.
    pop_result_list : bool
          Should unpack the result field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    pop_result_list : bool
          Should unpack the result field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    pop_begin_list : bool
          Should unpack the begin field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    pop_end_list : bool
          Should unpack the end field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    pop_embed_list : bool
          Should unpack the embed field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    pop_meta_list : bool
          Should unpack the meta field. Only set true for annotators that return exactly one element in their result, like Document classifier! This will convert list with just 1 element into just their element in the final pandas representation
    meta_white_list : List[str]
          Whitelist some keys which should be fetched from meta map. If this is not [], meta_black_list will be ignored
    meta_black_list : List[str]
          black_list some keys which should not be fetched from meta map
    meta_data_extractor : List[str]
          An extractor is a method that takes in [Dict[str,Any]] and returs [Any]
    begin_extractor : SparkNLPExtractor
          DOCS
    end_extractor : SparkNLPExtractor
          DOCS
    result_extractor : SparkNLPExtractor
          DOCS
    embedding_extractor : SparkNLPExtractor
          DOCS

    """
    ## TODO pretty __repr__ or __to__string() method! Leverage  SparkNLPExtractor fields
    output_col_prefix   :str
    get_positions       :bool              = field(default = False)
    get_begin           :bool              = field(default = False)
    get_end             :bool              = field(default = False)
    get_embeds          :bool              = field(default = False)
    get_result          :bool              = field(default = False)
    get_meta            :bool              = field(default = False)
    get_sentence_origin :bool              = field(default = False) # Should extract from which sentence a prediction was generated from. If output level is Document, this field is irrelevant and should be set to false
    get_full_meta       :bool              = field(default = False)
    get_annotator_type  :bool              = field(default = False)
    pop_result_list     :bool              = field(default = False) # TODO implement in ex
    pop_begin_list      :bool              = field(default = False) # TODO implement in ex
    pop_end_list        :bool              = field(default = False) # TODO implement in ex
    pop_embeds_list     :bool              = field(default = False) # TODO implement in ex
    pop_meta_list       :bool              = field(default = False) # TODO implement in ex
    pop_never           :bool              = field(default = False) # never ever pop
    meta_black_list     :List[str]         = field(default = list)
    meta_white_list     :List[str]         = field(default = list)
    meta_data_extractor :SparkNLPExtractor = field(default = SparkNLPExtractor())
    begin_extractor     :SparkNLPExtractor = field(default = SparkNLPExtractor())
    end_extractor       :SparkNLPExtractor = field(default = SparkNLPExtractor())
    result_extractor    :SparkNLPExtractor = field(default = SparkNLPExtractor())
    embedding_extractor :SparkNLPExtractor = field(default = SparkNLPExtractor())
    description         :str               = field(default = '')
    name                :str               = field(default = '')



from nlu.pipe.pipe_components import SparkNLUComponent
class Classifier(SparkNLUComponent):
    def __init__(self, annotator_class='sentiment_dl', language='en', component_type='classifier', get_default=True, model = None, nlp_ref ='', nlu_ref='',trainable=False, is_licensed=False, do_ref_checks=True, lang='en',loaded_from_pretrained_pipe=False):
        if do_ref_checks:
            if 'e2e' in nlu_ref or 'toxic' in nlu_ref : annotator_class= 'multi_classifier'
            elif 'e2e' in nlp_ref or 'toxic' in nlp_ref : annotator_class= 'multi_classifier'

            elif 'multiclassifierdl' in nlp_ref : annotator_class= 'multi_classifier'
            elif 'classifierdl' in nlp_ref: annotator_class= 'classifier_dl'

            elif 'yake' in nlu_ref: annotator_class= 'yake'
            elif 'yake' in nlp_ref: annotator_class= 'yake'

            elif 'sentimentdl' in nlp_ref : annotator_class= 'sentiment_dl'

            elif 'vivekn' in nlp_ref or 'vivekn' in nlp_ref : annotator_class= 'vivekn_sentiment'

            elif 'wiki_' in nlu_ref or 'wiki_' in nlp_ref : annotator_class= 'language_detector'
            elif 'pos' in nlu_ref and 'ner' not in nlu_ref:  annotator_class= 'pos'
            elif 'pos' in nlp_ref and 'ner' not in nlp_ref:  annotator_class= 'pos'

            elif 'icd' in nlu_ref: annotator_class= 'classifier_dl'
            elif 'med_ner' in nlu_ref: annotator_class= 'ner_healthcare'
            elif 'generic_classifier' in nlu_ref: annotator_class= 'generic_classifier'
            elif 'ner' in nlu_ref and 'generic' not in nlu_ref : annotator_class= 'ner'
            elif 'ner' in nlp_ref and 'generic' not in nlp_ref : annotator_class= 'ner'

        if model != None :
            self.model = model
            from sparknlp.annotator import NerDLModel,NerCrfModel
            if isinstance(self.model, (NerDLModel,NerCrfModel)): self.model.setIncludeConfidence(True)
            elif is_licensed :
                from sparknlp_jsl.annotator import MedicalNerModel
                if isinstance(self.model, MedicalNerModel): self.model.setIncludeConfidence(True)

        else :
            if 'sentiment' in annotator_class and 'vivekn' not in annotator_class:
                from nlu import SentimentDl
                if trainable : self.model = SentimentDl.get_default_trainable_model()
                elif is_licensed : self.model = SentimentDl.get_pretrained_model(nlp_ref, language, bucket='clinical/models')
                elif get_default : self.model = SentimentDl.get_default_model()
                else : self.model = SentimentDl.get_pretrained_model(nlp_ref, language)
            elif 'generic_classifier' in annotator_class:
                from nlu.components.classifiers.generic_classifier.generic_classifier import GenericClassifier
                if trainable : self.model     = GenericClassifier.get_default_trainable_model()
                else : self.model = GenericClassifier.get_pretrained_model(nlp_ref, language, bucket='clinical/models')
            elif 'vivekn' in annotator_class:
                from nlu import ViveknSentiment
                if get_default : self.model = ViveknSentiment.get_default_model()
                else : self.model = ViveknSentiment.get_pretrained_model(nlp_ref, language)
            elif 'ner' in annotator_class  and 'ner_healthcare' not in annotator_class:
                from nlu import NERDL
                if trainable : self.model = NERDL.get_default_trainable_model()
                elif is_licensed : self.model = NERDL.get_pretrained_model(nlp_ref, language, bucket='clinical/models')
                elif get_default : self.model = NERDL.get_default_model()
                else : self.model = NERDL.get_pretrained_model(nlp_ref, language)
                if hasattr(self, 'model'): self.model.setIncludeConfidence(True)

            elif 'ner.crf' in annotator_class:
                from nlu import NERDLCRF
                if get_default : self.model = NERDLCRF.get_default_model()
                else : self.model = NERDLCRF.get_pretrained_model(nlp_ref, language)
                if hasattr(self, 'model'): self.model.setIncludeConfidence(True)
            elif ('classifier_dl' in annotator_class or annotator_class == 'toxic') and not 'multi' in annotator_class:
                from nlu import ClassifierDl
                if trainable: self.model = ClassifierDl.get_trainable_model()
                elif is_licensed : self.model = ClassifierDl.get_pretrained_model(nlp_ref, language, bucket='clinical/models')
                elif get_default : self.model = ClassifierDl.get_default_model()
                else : self.model = ClassifierDl.get_pretrained_model(nlp_ref, language)
                if hasattr(self.model,'setIncludeConfidence'): self.model.setIncludeConfidence(True)

            elif 'language_detector' in annotator_class:
                from nlu import LanguageDetector
                if get_default : self.model = LanguageDetector.get_default_model()
                else: self.model = LanguageDetector.get_pretrained_model(nlp_ref, language)
            elif 'pos' in annotator_class:
                from nlu import PartOfSpeechJsl
                if trainable : self.model = PartOfSpeechJsl.get_default_trainable_model()
                elif get_default : self.model = PartOfSpeechJsl.get_default_model()
                elif is_licensed : self.model = PartOfSpeechJsl.get_pretrained_model(nlp_ref, language, bucket='clinical/models')
                else : self.model = PartOfSpeechJsl.get_pretrained_model(nlp_ref, language)

            elif 'yake' in annotator_class:
                from nlu import Yake
                self.model  = Yake.get_default_model()
            elif 'multi_classifier' in annotator_class :
                from nlu import MultiClassifier
                if trainable : self.model = MultiClassifier.get_default_trainable_model()
                elif get_default : self.model = MultiClassifier.get_default_model()
                else : self.model = MultiClassifier.get_pretrained_model(nlp_ref, language)
            elif 'ner_healthcare' in annotator_class:
                from nlu.components.classifiers.ner_healthcare.ner_dl_healthcare import NERDLHealthcare
                if trainable : self.model     = NERDLHealthcare.get_default_trainable_model()
                else : self.model = NERDLHealthcare.get_pretrained_model(nlp_ref, language, bucket='clinical/models')



        SparkNLUComponent.__init__(self, annotator_class, component_type, nlu_ref, nlp_ref, lang,loaded_from_pretrained_pipe , is_licensed)



from nlu.pipe.pipe_components import SparkNLUComponent

class Util(SparkNLUComponent):

    def __init__(self, annotator_class='document_assembler', component_type='util', model = None, loaded_from_pretrained_pipe = False, nlu_ref='',nlp_ref='',lang='en',is_licensed=False):
        # super(Tokenizer,self).__init__(annotator_class = annotator_class, component_type = component_type)
        if annotator_class == 'ner_converter':
            annotator_class = 'ner_to_chunk_converter'
        if model != None : self.model = model
        else :
            if annotator_class == 'document_assembler':
                from nlu import SparkNlpDocumentAssembler
                self.model =  SparkNlpDocumentAssembler.get_default_model()
            elif annotator_class == 'deep_sentence_detector' :
                from nlu import SentenceDetectorDeep
                self.model =  SentenceDetectorDeep.get_default_model()
            elif annotator_class == 'sentence_detector' :
                from nlu import SparkNLPSentenceDetector
                self.model =  SparkNLPSentenceDetector.get_default_model()
            elif annotator_class == 'ner_to_chunk_converter' :
                from nlu import NerToChunkConverter
                self.model =  NerToChunkConverter.get_default_model()
            elif annotator_class == 'sentence_embeddings':
                from nlu import SparkNLPSentenceEmbeddings
                self.model = SparkNLPSentenceEmbeddings.get_default_model()
            elif annotator_class == 'feature_assembler':
                from nlu.components.utils.feature_assembler.feature_assembler import SparkNLPFeatureAssembler
                self.model = SparkNLPFeatureAssembler.get_default_model()
            elif annotator_class == 'ner_to_chunk_converter_licensed':
                from nlu.components.utils.ner_to_chunk_converter_licensed.ner_to_chunk_converter_licensed import NerToChunkConverterLicensed
                self.model = NerToChunkConverterLicensed.get_default_model()
            elif annotator_class == 'chunk_merger':
                from nlu.components.utils.chunk_merger.chunk_merger import ChunkMerger
                self.model = ChunkMerger.get_default_model()
            elif annotator_class == 'doc2chunk':
                from nlu.components.utils.doc2chunk.doc_2_chunk import Doc_2_Chunk
                self.model = Doc_2_Chunk.get_default_model()


        SparkNLUComponent.__init__(self, annotator_class, component_type, nlu_ref, lang,loaded_from_pretrained_pipe )



import unittest
from tests.test_utils import get_sample_pdf_with_labels, get_sample_pdf, get_sample_sdf, get_sample_pdf_with_extra_cols, get_sample_pdf_with_no_text_col ,get_sample_spark_dataframe
from nlu import *


class TestNameSpace(unittest.TestCase):

    def test_tokenize(self):
        df = nlu.load('en.tokenize').predict('What a wonderful day!')
        
        print(df)

        df = nlu.load('tokenize').predict('What a wonderful day!')
        print(df)
        

    def test_pos(self):

        df = nlu.load('pos', verbose=True).predict('What a wonderful day!')
        print(df)
    # 
    # def test_embed(self):
    #     # df = nlu.load('en.embed').predict('What a wonderful day!')
    #     #
    #     # print(df)
    # 
    #     df = nlu.load('embed').predict('What a wonderful day!')
    #     print(df)
    #     
    # 
    # def test_embed_glove(self):
    #     df = nlu.load('en.embed.glove').predict('What a wonderful day!')
    #     
    #     print(df)
    # 
    #     df = nlu.load('embed.glove').predict('What a wonderful day!')
    #     print(df)
    #     df = nlu.load('glove').predict('What a wonderful day!')
    #     print(df)
    # 
    
    def test_sentiment_twitter_out(self):
        # res=nlu.load('en.sentiment.twitter',verbose=True).predict('@elonmusk Tesla stock price is too high imo') # ifninite loop ?? 
        res = nlu.load('en.sentiment.imdb',verbose=True).predict('The Matrix was a pretty good movie')

        print(res)
        print(res.columns)

    def test_output_levels(self):
        print('token test')
        df = nlu.load('sentiment',verbose=True).predict('What a wonderful day!', output_level='token')
        print(df)
        print('document test')
        df = nlu.load('sentiment',verbose=True).predict('What a wonderful day!', output_level='document')
        print(df)

        print('sentence test')
        df = nlu.load('sentiment',verbose=True).predict('What a wonderful day!', output_level='sentence')
        print(df)

        print('chunk test')
        df = nlu.load('sentiment',verbose=True).predict('I like peanut butter and jelly!', output_level='chunk')
        print(df)

    def test_ner_multilingual(self):
        df = nlu.load('ner',verbose=True).predict('New York is a great place and America aswell')

        print(df)

    def test_sentiment(self):
        df = nlu.load('en.sentiment').predict('What a wonderful day!')


    def test_emotion(self):
        df = nlu.load('en.classify.emotion').predict('What a wonderful day!')
        
        print(df)


    def test_spell(self):

        df = nlu.load('spell').predict('What a wonderful day!')
        print(df)
        
    #
    def test_dependency(self):
        df = nlu.load('dep', verbose=True).predict('What a wonderful day!')
        print(df)

    def test_dependency_untyped(self):
        df = nlu.load('dep.untyped', verbose=True).predict('What a wonderful day!')

        print(df)

    def test_bert(self):
        df = nlu.load('bert').predict('What a wonderful day!')

        print(df)


    def test_lang(self):
        df = nlu.load('lang', verbose=True).predict('What a wonderful day!')
        print(df)
        print(df.columns)
        print(df['language_de'])
        print(df['language_fr'])
        print(len(df['language_de'][0]))
        # df = nlu.load('xx.classify.lang').predict('What a wonderful day!')
        # print(df)
        # df = nlu.load('classify.lang').predict('What a wonderful day!')
        # print(df)
        # print(df)


    def test_explain(self):
        df = nlu.load('en.explain').predict('What a wonderful day!')
        print(df)
        df = nlu.load('explain').predict('What a wonderful day!')
        print(df)


    def test_match(self):
        df = nlu.load('match.date',verbose=True).predict('What a wonderful day!')
        print(df)
        # df = nlu.load('en.match.date').predict('What a wonderful day!')
        # print(df)

    def test_clean_stop(self):
        # df = nlu.load('clean.stop').predict('What a wonderful day!')
        # print(df)
        df = nlu.load('en.clean.stop').predict('What a wonderful day!')
        print(df)


    def test_spell(self):
        df = nlu.load('spell').predict('What a wonderful day!')

        print(df)

        df = nlu.load('en.spell').predict('What a wonderful day!')

        print(df)

    # def test_all_spell(self):
        # df = nlu.load('en.spell.symmetric').predict('What a wonderful day!')
        #
        # print(df)
        #
        # df = nlu.load('en.spell.context').predict('What a wonderful day!')

        # print(df)
        # df = nlu.load('en.spell.norvig').predict('What a wonderful day!')
        # 
        # print(df)
        # df = nlu.load('spell').predict('What a wonderful day!')
        # 
        # print(df)
        # 
        # df = nlu.load('en.spell').predict('What a wonderful day!')
        # 
        # print(df)

    # def test_biobert(self):
    #     df = nlu.load('biobert').predict('What a wonderful day!')
    # 
    #     print(df)
    # 
    #     df = nlu.load('en.embed.biobert').predict('What a wonderful day!')
    #     print(df)
    # 
    # def test_elmo(self):
    #     df = nlu.load('en.embed.elmo').predict('What a wonderful day!')
    #     print(df)
    #     df = nlu.load('elmo').predict('What a wonderful day!')
    #     print(df)
    # 
    # def test_use(self):
    #     df = nlu.load('en.embed.use').predict('What a wonderful day!')
    # 
    #     print(df)
    # 
    #     df = nlu.load('use').predict('What a wonderful day!')
    #     print(df)
    # 
    # def test_albert(self):
    #     df = nlu.load('en.embed.albert').predict('What a wonderful day!')
    # 
    #     print(df)
    # 
    #     df = nlu.load('albert').predict('What a wonderful day!')
    #     print(df)
    # 
    # def test_xlnet(self):
    #     df = nlu.load('en.embed.xlnet').predict('What a wonderful day!')
    # 
    #     print(df)
    # 
    #     df = nlu.load('xlnet').predict('What a wonderful day!')
    #     print(df)

    def test_lemma(self):
        df = nlu.load('lemma').predict('What a wonderful day!')

        print(df)
        df = nlu.load('en.lemma').predict('What a wonderful day!')

        print(df)

    # def test_norm(self):
    #     df = nlu.load('lemma').predict('What a wonderful day!')
    #
    #     print(df)
    #     df = nlu.load('en.lemma').predict('What a wonderful day!')
    #
    #     print(df)
    # 
    # def test_use(self):
    #     df = nlu.load('en.embed_sentence.use').predict('What a wonderful day!')
    #     print(df)
    # 
    # def test_glove(self):
    #     df = nlu.load('nl.ner.wikiner.glove_6B_300').predict('What a wonderful day!')
    # 
    #     print(df)

    def test_sentence_detector(self):
        df = nlu.load('sentence_detector', verbose=True).predict('What a wonderful day! Tomorrow will be even better!')

        print(df)

    def test_stopwords(self):
        df = nlu.load('match.chunk').predict('What a wonderful day!')
        print(df)

    def test_classify_lang(self):
        df = nlu.load('xx.classify.wiki_7').predict('What a wonderful day!')
        print(df)

    def test_sentiment_on_datasets(self):
        df = nlu.load('sentiment.twitter').predict('What a wonderful day!')
        print(df)
        # df = nlu.load('sentiment.imdb').predict('What a wonderful day!')
        # print(df)

    def test_multiple_nlu_references(self):
        # df = nlu.load('elmo bert').predict('What a wonderful day!')
        df = nlu.load('elmo').predict('What a wonderful day!')

        print(df)
        # df = nlu.load('sentiment.imdb').predict('What a wonderful day!')
        # print(df)
        
    def test_sentiment_output(self):
        res = nlu.load('sentiment',verbose=True).predict('Your life is the sum of a remainder of an unbalanced equation inherent to the programming of the matrix. You are the eventuality of an anomaly, which despite my sincerest efforts I have been unable to eliminate from what is otherwise a harmony of mathematical precision. While it remains a burden assiduously avoided, it is not unexpected, and thus not beyond a measure of control. Which has led you, inexorably, here.', output_level='sentence')
        # res = nlu.load('bert',verbose=True).predict('@Your life is the sum of a remainder of an unbalanced equation inherent to the programming of the matrix. You are the eventuality of an anomaly, which despite my sincerest efforts I have been unable to eliminate from what is otherwise a harmony of mathematical precision. While it remains a burden assiduously avoided, it is not unexpected, and thus not beyond a measure of control. Which has led you, inexorably, here.', output_level='sentence')
        
        print(res)
        print(res['sentiment'])

        print(res.dtypes)

    def test_stem(self):
        pdf = get_sample_pdf()
        res = nlu.load('stem',verbose=True).predict(pdf )
        print(res)
        res = nlu.load('en.stem',verbose=True).predict(pdf)
        print(res)

    def test_norm(self):
        pdf = get_sample_pdf()
        res = nlu.load('norm',verbose=True).predict(pdf, output_positions=True )
        print(res)
        # res = nlu.load('en.norm',verbose=True).predict(pdf)
        # print(res)

    def test_chunk(self):
        res = nlu.load('chunk',verbose=True).predict('I like peanut butter and jelly!' )
        print(res)        

    def test_ngram(self):
        pdf = get_sample_pdf()
        # res = nlu.load('ngram',verbose=True).predict(pdf )
        pipe = nlu.load('ngram',verbose=True)
        # print(res['ngrams'])
        print("PIPE", pipe)
        res = nlu.load('en.ngram',verbose=True).predict(pdf)
        print(res['ngrams'])
    

    def test_chunk_embeds(self):
        pdf = get_sample_pdf()
        res = nlu.load('embed_chunk',verbose=True).predict("What a wondful day!" )
        print(res)
        res = nlu.load('en.embed_chunk',verbose=True).predict(pdf)
        print(res)

    def test_regex_matcher(self):
        pdf = get_sample_pdf()
        res = nlu.load('match.regex',verbose=True).predict(pdf )
        print(res)

    def test_text_matcher(self):
        pdf = get_sample_pdf()
        res = nlu.load('match.text',verbose=True).predict(pdf )
        print(res)

    def test_auto_sentence_embed_bert(self): # TODO WIP
        pdf = get_sample_pdf()
        res = nlu.load('embed_sentence.bert',verbose=True).predict(pdf )
        print(res)

    def test_auto_sentence_embed_elmo(self): # TODO  WIP
        pdf = get_sample_pdf()
        res = nlu.load('embed_sentence.elmo',verbose=True).predict(pdf )
        print(res)
        
        
    # def test_bad_pandas_column_datatype(self):
    #     sdf = get_sample_spark_dataframe()
    #     res = nlu.load('asdasj.asdas',verbose=True).predict(sdf, output_level='sentence')
    #     # res = nlu.load('bert',verbose=True).predict('@Your life is the sum of a remainder of an unbalanced equation inherent to the programming of the matrix. You are the eventuality of an anomaly, which despite my sincerest efforts I have been unable to eliminate from what is otherwise a harmony of mathematical precision. While it remains a burden assiduously avoided, it is not unexpected, and thus not beyond a measure of control. Which has led you, inexorably, here.', output_level='sentence')
    # 
    #     print(res)
    # 
    # def test_bad_pandas_dataframe_datatype(self):
    #     sdf = get_sample_spark_dataframe()
    #     res = nlu.load('asdasj.asdas',verbose=True).predict(sdf, output_level='sentence')
    #     # res = nlu.load('bert',verbose=True).predict('@Your life is the sum of a remainder of an unbalanced equation inherent to the programming of the matrix. You are the eventuality of an anomaly, which despite my sincerest efforts I have been unable to eliminate from what is otherwise a harmony of mathematical precision. While it remains a burden assiduously avoided, it is not unexpected, and thus not beyond a measure of control. Which has led you, inexorably, here.', output_level='sentence')
    # 
    #     print(res)

    #2.6 test


    def test_electra(self):
        pdf = get_sample_pdf()
        res = nlu.load('en.embed.electra',verbose=True).predict(pdf )
        print(res)

    def test_embed_sentence_bert(self):
        pdf = get_sample_pdf()
        res = nlu.load('en.embed_sentence.small_bert_L2_128',verbose=True).predict(pdf )
        print(res)

    def test_embed_sentence_bert(self):
        pdf = get_sample_pdf()
        res = nlu.load('en.embed_sentence.biobert.pubmed_base_cased',verbose=True).predict(pdf )
        print(res)


    def test_toxic(self):
        pdf = get_sample_pdf()
        res = nlu.load('en.classify.toxic',verbose=True).predict(pdf )
        print(res)

    def test_e2e(self):
        pdf = get_sample_pdf()
        res = nlu.load('en.classify.e2e',verbose=True).predict(pdf )
        print(res)



    def test_labse(self):
        pdf = get_sample_pdf()
        res = nlu.load('xx.embed_sentence.labse',verbose=True).predict(pdf )
        print(res)

    def test_xx_bert(self):
        pdf = get_sample_pdf()
        res = nlu.load('xx.embed_sentence',verbose=True).predict(pdf )
        print(res)
        
    def test_26_bert(self):
        res = nlu.load('en.ner.bert',verbose=True).predict('The NLU library is a machine learning library, simmilar to Tensorflow and Keras')
        print(res)



if __name__ == '__main__':
    unittest.main()




import unittest
from tests.test_utils import get_sample_pdf_with_labels, get_sample_pdf, get_sample_sdf, get_sample_pdf_with_extra_cols, get_sample_pdf_with_no_text_col ,get_sample_spark_dataframe
from nlu import *

class TestSentenceDetector(unittest.TestCase):

    def test_sentence_detector(self):
        pipe = nlu.load('sentence_detector', verbose=True , )
        df = pipe.predict('I like my sentences detected. Some like their sentences warm. Warm is also good.', output_level='sentence',drop_irrelevant_cols=False, metadata=True, )
        for c in df.columns: print(df[c])


    def test_sentence_detector_multi_lang(self):
        pipe = nlu.load('xx.sentence_detector', verbose=True , )
        df = pipe.predict('I like my sentences detected. Some like their sentences warm. Warm is also good.', output_level='sentence',drop_irrelevant_cols=False, metadata=True, )
        for c in df.columns: print(df[c])


    def test_sentence_detector_pragmatic(self):
        pipe = nlu.load('sentence_detector.pragmatic', verbose=True , )
        df = pipe.predict('I like my sentences detected. Some like their sentences warm. Warm is also good.', output_level='sentence',drop_irrelevant_cols=False, metadata=True, )
        for c in df.columns: print(df[c])


if __name__ == '__main__':
    unittest.main()




from nlu.spellbook import Spellbook
import nlu
all_components_info = nlu.AllComponentsInfo()

class Discoverer:
    """Various methods that help discover nlu_refs and functionality"""
    def __init__(self):
        ''' Initialize every NLU component info object and provide access to them'''
        self.nlu_info = {}

    @staticmethod
    def get_components(m_type='', include_pipes=False, lang='', licensed=False, get_all=False,include_aliases=True):
        """Filter all NLU components

        m_type : Component/Model type to filter for
        include_pipes : Weather to include pipelines in the result or not
        lang : Which languages to include. By default lang='' will get every lang
        licensed : Wether to include licensed models or not
        get_all: If set to true, will ignore other params and gets EVERY NLU_ref from defined name spaces
        """
        nlu_refs_of_type = []
        model_universe = nlu.Spellbook.pretrained_models_references
        for lang_, models in model_universe.items():
            if lang != '' :
                if lang_!= lang : continue
            for nlu_ref, nlp_ref in model_universe[lang_].items():
                if m_type in nlu_ref or get_all: nlu_refs_of_type.append(nlu_ref)

        if include_pipes :
            model_universe = nlu.Spellbook.pretrained_pipe_references
            for lang_, models in model_universe.items():
                if lang != '':
                    if lang_!= lang : continue
                for nlu_ref, nlp_ref in model_universe[lang_].items():
                    if m_type in nlu_ref or get_all: nlu_refs_of_type.append(nlu_ref)
        if include_aliases :
            model_universe = nlu.Spellbook.component_alias_references
            for nlu_ref, nlp_ref in model_universe.items():
                if m_type in nlu_ref or get_all: nlu_refs_of_type.append(nlu_ref)

        if licensed:
            model_universe = nlu.Spellbook.pretrained_healthcare_model_references
            for lang_, models in model_universe.items():
                if lang != '':
                    if lang_!= lang : continue
                for nlu_ref, nlp_ref in model_universe[lang_].items():
                    if m_type in nlu_ref or get_all: nlu_refs_of_type.append(nlu_ref)

        return list(set(nlu_refs_of_type))





    @staticmethod
    def print_all_languages():
        ''' Print all languages which are available in NLU Spark NLP pointer '''
        print('Languages available in NLU : \n ')
        for lang in all_components_info.all_languages: print(lang)



    @staticmethod
    def print_all_nlu_components_for_lang(lang='en', c_type='classifier'):
        #todo parse for lang
        '''Print all NLU components available for a language Spark NLP pointer'''
        if lang in all_components_info.all_languages:
            # print("All Pipelines for language"+ lang+ "\n"+)
            for nlu_reference in Spellbook.pretrained_pipe_references[lang]:
                print("nlu.load('" + nlu_reference + "') returns Spark NLP Pipeline:" +
                      Spellbook.pretrained_pipe_references[lang][nlu_reference])

            for nlu_reference in Spellbook.pretrained_models_references[lang]:
                print("nlu.load('" + nlu_reference + "') returns Spark NLP Model: " +
                      Spellbook.pretrained_models_references[lang][nlu_reference])

        else:
            print(
                "Language " + lang + " Does not exsist in NLU. Please check the docs or nlu.print_all_languages() for supported language references")


    @staticmethod
    def print_components(lang='', action=''):
        '''
        Print every single NLU reference for models and pipeliens and their Spark NLP pointer
        :param lang: Language requirements for the components filterd. See nlu.languages() for supported languages
        :param action: Components that will be filterd.
        :return: None. This method will print its results.
        '''
        if lang != '' and action == '':
            nlu.Discoverer().print_all_nlu_components_for_lang(lang)
            return

        if lang != '' and action != '':
            nlu.Discoverer().print_all_model_kinds_for_action_and_lang(lang, action)
            return

        if lang == '' and action != '':
            nlu.Discoverer().print_all_model_kinds_for_action(action)
            return

        # Print entire Namespace below
        for nlu_reference in nlu.Spellbook.component_alias_references.keys():
            component_type = nlu.Spellbook.component_alias_references[nlu_reference][1][0],  # pipe or model
            print("nlu.load('" + nlu_reference + "') '  returns Spark NLP " + str(component_type) + ': ' +
                  nlu.Spellbook.component_alias_references[nlu_reference][0])

        for lang in nlu.Spellbook.pretrained_pipe_references.keys():
            for nlu_reference in nlu.Spellbook.pretrained_pipe_references[lang]:
                print("nlu.load('" + nlu_reference + "') for lang" + lang + " returns model Spark NLP model:" +
                      nlu.Spellbook.pretrained_pipe_references[lang][nlu_reference])

        for lang in nlu.Spellbook.pretrained_models_references.keys():
            for nlu_reference in nlu.Spellbook.pretrained_models_references[lang]:
                print("nlu.load('" + nlu_reference + "')' for lang" + lang + " returns model Spark NLP model: " +
                      nlu.Spellbook.pretrained_models_references[lang][nlu_reference])


    @staticmethod
    def print_component_types():
        ''' Prints all unique component types in NLU'''
        types = []
        for key, val in nlu.all_components_info.all_components.items(): types.append(val.type)

        types = set(types)
        print("Provided component types in this NLU version are : ")
        for i, type in enumerate(types):
            print(i, '. ', type)


    @staticmethod
    def print_all_model_kinds_for_action(action):
        for lang, lang_models in nlu.Spellbook.pretrained_models_references.items():
            lang_printed = False
            for nlu_reference, nlp_reference in lang_models.items():
                ref_action = nlu_reference.split('.')
                if len(ref_action) > 1:
                    ref_action = ref_action[1]

                if ref_action == action:
                    if not lang_printed:
                        print('For language <' + lang + '> NLU provides the following Models : ')
                        lang_printed = True
                    print("nlu.load('" + nlu_reference + "') returns Spark NLP model " + nlp_reference)


    @staticmethod
    def print_all_model_kinds_for_action_and_lang(lang, action):
        lang_candidates = nlu.Spellbook.pretrained_models_references[lang]
        print("All NLU components for lang ", lang, " that peform action ", action)
        for nlu_reference, nlp_reference in lang_candidates.items():
            ref_action = nlu_reference.split('.')
            if len(ref_action) > 1: ref_action = ref_action[1]
            if ref_action == action:
                print("nlu.load('" + nlu_reference + "') returns Spark NLP model " + nlp_reference)

    @staticmethod
    def print_trainable_components():
        '''
        # todo update
        Print every trainable Algorithm/Model
        :return: None
        '''
        i = 1
        print('The following models can be trained with a dataset that provides a label column and matching dataset')
        for name, infos in nlu.all_components_info.all_components.items() :
            if infos.trainable == True :
                print(f' {i}. {name}')
                i+=1



from nlu.utils.environment.env_utils import *







def install_and_import_healthcare(JSL_SECRET):
    """ Install Spark-NLP-Healthcare PyPI Package in current enviroment if it cannot be imported and liscense provided"""
    import importlib
    try:
        importlib.import_module('sparknlp_jsl')
    except ImportError:
        import pip
        print("Spark NLP Healthcare could not be imported. Installing latest spark-nlp-jsl PyPI package via pip...")
        hc_version      = JSL_SECRET.split('-')[0]
        import pyspark
        pip_major_version = int(pip.__version__.split('.')[0])
        if pip_major_version in [10, 18, 19,20]:
            # for these versions pip module does not support installing, we install via OS command.
            os.system(f'pip install spark-nlp-jsl=={hc_version} --extra-index-url https://pypi.johnsnowlabs.com/{JSL_SECRET}')
        else:
            pip.main(['install', f'spark-nlp-jsl=={hc_version}', '--extra-index-url', f'https://pypi.johnsnowlabs.com/{JSL_SECRET}'])
    finally:
        import site
        from importlib import reload
        reload(site)
        globals()['sparknlp_jsl'] = importlib.import_module('sparknlp_jsl')







def authenticate_enviroment(SPARK_NLP_LICENSE,AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY):
    """Set Secret environ variables for Spark Context"""
    os.environ['SPARK_NLP_LICENSE'] = SPARK_NLP_LICENSE
    os.environ['AWS_ACCESS_KEY_ID']= AWS_ACCESS_KEY_ID
    os.environ['AWS_SECRET_ACCESS_KEY'] = AWS_SECRET_ACCESS_KEY

def get_authenticated_spark(SPARK_NLP_LICENSE,AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY,JSL_SECRET, gpu=False,):
    """
    Authenticates enviroment if not already done so and returns Spark Context with Healthcare Jar loaded
    0. If no Spark-NLP-Healthcare, install it via PyPi
    1. If not auth, run authenticate_enviroment()

    """

    authenticate_enviroment(SPARK_NLP_LICENSE,AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY)
    install_and_import_healthcare(JSL_SECRET)


    import sparknlp_jsl
    if is_env_pyspark_2_3() : return sparknlp_jsl.start(JSL_SECRET, spark23=True, gpu=gpu)
    if is_env_pyspark_2_4() : return sparknlp_jsl.start(JSL_SECRET, spark24=True, gpu=gpu)
    if is_env_pyspark_3_0() or is_env_pyspark_3_1()  : return sparknlp_jsl.start(JSL_SECRET, gpu=gpu, public='3.1.0')
    print(f"Current Spark version {get_pyspark_version()} not supported!")
    raise ValueError

def is_authorized_enviroment():
    """Check if auth secrets are set in enviroment"""
    SPARK_NLP_LICENSE = os.getenv('SPARK_NLP_LICENSE')
    AWS_ACCESS_KEY_ID = os.getenv('AWS_ACCESS_KEY_ID')
    AWS_SECRET_ACCESS_KEY = os.getenv('AWS_SECRET_ACCESS_KEY')
    return None not in [SPARK_NLP_LICENSE,AWS_ACCESS_KEY_ID,AWS_SECRET_ACCESS_KEY]



from sparknlp_display import NerVisualizer,DependencyParserVisualizer
from sparknlp.annotator import NerConverter,DependencyParserModel, TypedDependencyParserModel, PerceptronModel
from sparknlp.base import  DocumentAssembler
class VizUtilsOS():
    """Utils for interfacing with the Spark-NLP-Display lib and vizzing Open Source Components - Open source"""
    HTML_WRAPPER = """<div class="scroll entities" style="overflow-x: auto; border: 1px solid #e6e9ef; border-radius: 0.25rem; padding: 1rem; margin-bottom: 2.5rem; white-space:pre-wrap">{}</div>"""
    @staticmethod
    def infer_viz_open_source(pipe)->str:
        """For a given NLUPipeline with only open source components, infers which visualizations are applicable. """
        for c in pipe.components:
            if isinstance(c.model, NerConverter) : return 'ner'
            if isinstance(c.model, DependencyParserModel) : return 'dep'
    @staticmethod
    def viz_ner(anno_res, pipe,labels = None ,  viz_colors={},is_databricks_env=False,write_to_streamlit=False,streamlit_key='RANDOM' ):
        """Infer columns required for ner viz and then viz it.
        viz_colors :  set label colors by specifying hex codes , i.e. viz_colors =  {'LOC':'#800080', 'PER':'#77b5fe'}
        labels : only allow these labels to be displayed. (default: [] - all labels will be displayed)
        """
        document_col,entities_col =  VizUtilsOS.infer_ner_dependencies(pipe)
        ner_vis = NerVisualizer()
        ner_vis.set_label_colors(viz_colors)
        if write_to_streamlit :
            import streamlit as st
            HTML = ner_vis.display(anno_res,label_col=entities_col,document_col=document_col, labels=labels, return_html=True )
            CSS,HTML = HTML.split('</style>')
            CSS = CSS + '</style>'
            HTML = f'<div> {HTML} '
            st.markdown(CSS, unsafe_allow_html=True)
            st.markdown(VizUtilsOS.HTML_WRAPPER.format(HTML), unsafe_allow_html=True)

        elif not is_databricks_env:
            ner_vis.display(anno_res,label_col=entities_col,document_col=document_col, labels=labels )
        else :
            return ner_vis.display(anno_res,label_col=entities_col,document_col=document_col, labels=labels, return_html=True )

    @staticmethod
    def infer_ner_dependencies(pipe):
        """Finds entities and doc cols for ner viz"""
        doc_component      = None
        entities_component = None
        for c in pipe.components:
            if isinstance(c.model, NerConverter) :        entities_component  = c
            if isinstance(c.model, DocumentAssembler) :   doc_component = c

        document_col     = doc_component.info.outputs[0]
        entities_col = entities_component.info.outputs[0]
        return document_col, entities_col


    @staticmethod
    def viz_dep(anno_res,pipe,is_databricks_env,write_to_streamlit,streamlit_key='RANDOM'):
        """Viz dep result"""
        pos_col,dep_typ_col,dep_untyp_col  = VizUtilsOS.infer_dep_dependencies(pipe)
        dependency_vis = DependencyParserVisualizer()

        if write_to_streamlit :
            import streamlit as st
            SVG = dependency_vis.display(anno_res,pos_col =pos_col,dependency_col =  dep_untyp_col ,dependency_type_col = dep_typ_col,return_html=True)
            # st.markdown(SVG, unsafe_allow_html=True)
            st.markdown(VizUtilsOS.HTML_WRAPPER.format(SVG), unsafe_allow_html=True)

        elif not is_databricks_env:
            dependency_vis.display(anno_res,pos_col =pos_col,dependency_col =  dep_untyp_col ,dependency_type_col = dep_typ_col)
        else:
            return dependency_vis.display(anno_res,pos_col =pos_col,dependency_col =  dep_untyp_col ,dependency_type_col = dep_typ_col,return_html=True)

    @staticmethod
    def infer_dep_dependencies(pipe):
        """Finds entities,pos,dep_typed,dep_untyped and  doc cols for dep viz viz"""
        # doc_component      = None
        pos_component = None
        dep_untyped_component = None
        dep_typed_component = None
        for c in pipe.components:
            if isinstance(c.model, PerceptronModel) :              pos_component  = c
            if isinstance(c.model, TypedDependencyParserModel) :   dep_typed_component  = c
            if isinstance(c.model, DependencyParserModel) :        dep_untyped_component  = c

        pos_col       = pos_component.info.outputs[0]
        dep_typ_col   = dep_typed_component.info.outputs[0]
        dep_untyp_col = dep_untyped_component.info.outputs[0]
        return pos_col,dep_typ_col,dep_untyp_col





